@@ -34,6 +34,7 @@
 #include "psycopg/psycopg.h"
 #include "psycopg/cursor.h"
 #include "psycopg/connection.h"
+#include "psycopg/green.h"
 #include "psycopg/pqpath.h"
 #include "psycopg/typecast.h"
 #include "psycopg/microprotocols.h"
@@ -57,6 +58,7 @@ psyco_curs_close(cursorObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "")) return NULL;
 
     EXC_IF_CURS_CLOSED(self);
+    EXC_IF_ASYNC_IN_PROGRESS(self, close);
 
     if (self->name != NULL) {
         char buffer[128];
@@ -304,7 +306,7 @@ static PyObject *_psyco_curs_validate_sql_basic(
 }
 
 #define psyco_curs_execute_doc \
-"execute(query, vars=None, async=0) -- Execute query with bound vars."
+"execute(query, vars=None) -- Execute query with bound vars."
 
 static int
 _psyco_curs_execute(cursorObject *self,
@@ -313,19 +315,6 @@ _psyco_curs_execute(cursorObject *self,
     int res = 0;
     PyObject *fquery, *cvt = NULL;
 
-    Py_BEGIN_ALLOW_THREADS;
-    pthread_mutex_lock(&(self->conn->lock));
-    if (self->conn->async_cursor != NULL
-        && self->conn->async_cursor != (PyObject*)self) {
-        pthread_mutex_unlock(&(self->conn->lock));
-        Py_BLOCK_THREADS;
-        psyco_set_error(ProgrammingError, (PyObject*)self,
-                         "asynchronous query already in execution", NULL, NULL);
-        return 0;
-    }
-    pthread_mutex_unlock(&(self->conn->lock));
-    Py_END_ALLOW_THREADS;
-
     operation = _psyco_curs_validate_sql_basic(self, operation);
 
     /* Any failure from here forward should 'goto fail' rather than 'return 0'
@@ -454,13 +443,12 @@ _psyco_curs_execute(cursorObject *self,
 static PyObject *
 psyco_curs_execute(cursorObject *self, PyObject *args, PyObject *kwargs)
 {
-    long int async = 0;
     PyObject *vars = NULL, *operation = NULL;
 
-    static char *kwlist[] = {"query", "vars", "async", NULL};
+    static char *kwlist[] = {"query", "vars", NULL};
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O|Ol", kwlist,
-                                     &operation, &vars, &async)) {
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O|O", kwlist,
+                                     &operation, &vars)) {
         return NULL;
     }
 
@@ -484,8 +472,9 @@ psyco_curs_execute(cursorObject *self, PyObject *args, PyObject *kwargs)
     }
 
     EXC_IF_CURS_CLOSED(self);
+    EXC_IF_ASYNC_IN_PROGRESS(self, execute);
 
-    if (_psyco_curs_execute(self, operation, vars, async)) {
+    if (_psyco_curs_execute(self, operation, vars, self->conn->async)) {
         Py_INCREF(Py_None);
         return Py_None;
     }
@@ -515,6 +504,7 @@ psyco_curs_executemany(cursorObject *self, PyObject *args, PyObject *kwargs)
     }
 
     EXC_IF_CURS_CLOSED(self);
+    EXC_IF_CURS_ASYNC(self, executemany);
 
     if (self->name != NULL) {
         psyco_set_error(ProgrammingError, (PyObject*)self,
@@ -535,7 +525,7 @@ psyco_curs_executemany(cursorObject *self, PyObject *args, PyObject *kwargs)
         }
         else {
             if (self->rowcount == -1)
-		rowcount = -1;
+                rowcount = -1;
             else if (rowcount >= 0)
                 rowcount += self->rowcount;
             Py_DECREF(v);
@@ -544,8 +534,13 @@ psyco_curs_executemany(cursorObject *self, PyObject *args, PyObject *kwargs)
     Py_XDECREF(iter);
     self->rowcount = rowcount;
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    if (!PyErr_Occurred()) {
+        Py_INCREF(Py_None);
+        return Py_None;
+    }
+    else {
+        return NULL;
+    }
 }
 
 
@@ -656,23 +651,7 @@ _psyco_curs_prefetch(cursorObject *self)
 {
     int i = 0;
 
-    /* check if the fetching cursor is the one that did the asynchronous query
-       and raise an exception if not */
-    Py_BEGIN_ALLOW_THREADS;
-    pthread_mutex_lock(&(self->conn->lock));
-    if (self->conn->async_cursor != NULL
-        && self->conn->async_cursor != (PyObject*)self) {
-        pthread_mutex_unlock(&(self->conn->lock));
-        Py_BLOCK_THREADS;
-        psyco_set_error(ProgrammingError, (PyObject*)self,
-                         "asynchronous fetch by wrong cursor", NULL, NULL);
-        return -2;
-    }
-    pthread_mutex_unlock(&(self->conn->lock));
-    Py_END_ALLOW_THREADS;
-
-    if (self->pgres == NULL || self->needsfetch) {
-        self->needsfetch = 0;
+    if (self->pgres == NULL) {
         Dprintf("_psyco_curs_prefetch: trying to fetch data");
         do {
             i = pq_fetch(self);
@@ -767,7 +746,8 @@ psyco_curs_fetchone(cursorObject *self, PyObject *args)
 
     if (args && !PyArg_ParseTuple(args, "")) return NULL;
 
-    EXC_IF_CURS_CLOSED(self)
+    EXC_IF_CURS_CLOSED(self);
+    EXC_IF_ASYNC_IN_PROGRESS(self, fetchone);
     if (_psyco_curs_prefetch(self) < 0) return NULL;
     EXC_IF_NO_TUPLES(self);
 
@@ -828,6 +808,7 @@ psyco_curs_fetchmany(cursorObject *self, PyObject *args, PyObject *kwords)
     }
 
     EXC_IF_CURS_CLOSED(self);
+    EXC_IF_ASYNC_IN_PROGRESS(self, fetchmany);
     if (_psyco_curs_prefetch(self) < 0) return NULL;
     EXC_IF_NO_TUPLES(self);
 
@@ -900,6 +881,7 @@ psyco_curs_fetchall(cursorObject *self, PyObject *args)
     }
 
     EXC_IF_CURS_CLOSED(self);
+    EXC_IF_ASYNC_IN_PROGRESS(self, fetchall);
     if (_psyco_curs_prefetch(self) < 0) return NULL;
     EXC_IF_NO_TUPLES(self);
 
@@ -949,25 +931,25 @@ psyco_curs_fetchall(cursorObject *self, PyObject *args)
 /* callproc method - execute a stored procedure */
 
 #define psyco_curs_callproc_doc \
-"callproc(procname, parameters=None, async=0) -- Execute stored procedure."
+"callproc(procname, parameters=None) -- Execute stored procedure."
 
 static PyObject *
 psyco_curs_callproc(cursorObject *self, PyObject *args, PyObject *kwargs)
 {
     const char *procname = NULL;
     char *sql = NULL;
-    long int async = 0;
     Py_ssize_t procname_len, i, nparameters = 0, sl = 0;
     PyObject *parameters = Py_None;
     PyObject *operation = NULL;
     PyObject *res = NULL;
 
-    if (!PyArg_ParseTuple(args, "s#|Ol",
-          &procname, &procname_len, &parameters, &async
+    if (!PyArg_ParseTuple(args, "s#|O",
+          &procname, &procname_len, &parameters
        ))
     { return NULL; }
 
     EXC_IF_CURS_CLOSED(self);
+    EXC_IF_ASYNC_IN_PROGRESS(self, callproc);
 
     if (self->name != NULL) {
         psyco_set_error(ProgrammingError, (PyObject*)self,
@@ -995,7 +977,7 @@ psyco_curs_callproc(cursorObject *self, PyObject *args, PyObject *kwargs)
     operation = PyString_FromString(sql);
     PyMem_Free((void*)sql);
 
-    if (_psyco_curs_execute(self, operation, parameters, async)) {
+    if (_psyco_curs_execute(self, operation, parameters, self->conn->async)) {
         Py_INCREF(parameters);
         res = parameters;
     }
@@ -1084,6 +1066,7 @@ psyco_curs_scroll(cursorObject *self, PyObject *args, PyObject *kwargs)
         return NULL;
 
     EXC_IF_CURS_CLOSED(self);
+    EXC_IF_ASYNC_IN_PROGRESS(self, scroll)
 
     /* if the cursor is not named we have the full result set and we can do
        our own calculations to scroll; else we just delegate the scrolling
@@ -1168,6 +1151,11 @@ static int _psyco_curs_copy_columns(PyObject *columns, char *columnlist)
     }
     Py_DECREF(coliter);
 
+    /* Error raised by the coliter generator */
+    if (PyErr_Occurred()) {
+        return -1;
+    }
+
     if (offset == 2) {
         return 0;
     }
@@ -1228,6 +1216,8 @@ psyco_curs_copy_from(cursorObject *self, PyObject *args, PyObject *kwargs)
         return NULL;
 
     EXC_IF_CURS_CLOSED(self);
+    EXC_IF_CURS_ASYNC(self, copy_from);
+    EXC_IF_GREEN(copy_from);
 
     quoted_delimiter = psycopg_escape_string((PyObject*)self->conn, sep, 0, NULL, NULL);
     if (quoted_delimiter == NULL) {
@@ -1332,6 +1322,9 @@ psyco_curs_copy_to(cursorObject *self, PyObject *args, PyObject *kwargs)
         return NULL;
 
     EXC_IF_CURS_CLOSED(self);
+    EXC_IF_CURS_ASYNC(self, copy_to);
+    EXC_IF_GREEN(copy_to);
+
     quoted_delimiter = psycopg_escape_string((PyObject*)self->conn, sep, 0, NULL, NULL);
     if (quoted_delimiter == NULL) {
         PyErr_NoMemory();
@@ -1415,6 +1408,8 @@ psyco_curs_copy_expert(cursorObject *self, PyObject *args, PyObject *kwargs)
     { return NULL; }
 
     EXC_IF_CURS_CLOSED(self);
+    EXC_IF_CURS_ASYNC(self, copy_expert);
+    EXC_IF_GREEN(copy_expert);
 
     sql = _psyco_curs_validate_sql_basic(self, sql);
     
@@ -1461,71 +1456,6 @@ psyco_curs_copy_expert(cursorObject *self, PyObject *args, PyObject *kwargs)
     return res;
 }
 
-/* extension: fileno - return the file descripor of the connection */
-
-#define psyco_curs_fileno_doc \
-"fileno() -> int -- Return file descriptor associated to database connection."
-
-static PyObject *
-psyco_curs_fileno(cursorObject *self, PyObject *args)
-{
-    long int socket;
-
-    if (!PyArg_ParseTuple(args, "")) return NULL;
-    EXC_IF_CURS_CLOSED(self);
-
-    /* note how we call PQflush() to make sure the user will use
-       select() in the safe way! */
-    Py_BEGIN_ALLOW_THREADS;
-    pthread_mutex_lock(&(self->conn->lock));
-    PQflush(self->conn->pgconn);
-    socket = (long int)PQsocket(self->conn->pgconn);
-    pthread_mutex_unlock(&(self->conn->lock));
-    Py_END_ALLOW_THREADS;
-
-    return PyInt_FromLong(socket);
-}
-
-/* extension: isready - return true if data from async execute is ready */
-
-#define psyco_curs_isready_doc \
-"isready() -> bool -- Return True if data is ready after an async query."
-
-static PyObject *
-psyco_curs_isready(cursorObject *self, PyObject *args)
-{
-    int res;
-    
-    if (!PyArg_ParseTuple(args, "")) return NULL;
-    EXC_IF_CURS_CLOSED(self);
-
-    /* pq_is_busy does its own locking, we don't need anything special but if
-       the cursor is ready we need to fetch the result and free the connection
-       for the next query. if -1 is returned we raise an exception. */
-
-    res = pq_is_busy(self->conn);
-    
-    if (res == 1) {
-        Py_INCREF(Py_False);
-        return Py_False;
-    }
-    else if (res == -1) {
-        return NULL;
-    }
-    else {
-        IFCLEARPGRES(self->pgres);
-        Py_BEGIN_ALLOW_THREADS;
-        pthread_mutex_lock(&(self->conn->lock));
-        self->pgres = PQgetResult(self->conn->pgconn);
-        self->conn->async_cursor = NULL;
-        pthread_mutex_unlock(&(self->conn->lock));
-        Py_END_ALLOW_THREADS;
-        self->needsfetch = 1;
-        Py_INCREF(Py_True);
-        return Py_True;
-    }
-}
-
 /* extension: closed - return true if cursor is closed*/
 
 #define psyco_curs_closed_doc \
@@ -1604,10 +1534,6 @@ static struct PyMethodDef cursorObject_methods[] = {
 #ifdef PSYCOPG_EXTENSIONS
     {"mogrify", (PyCFunction)psyco_curs_mogrify,
      METH_VARARGS|METH_KEYWORDS, psyco_curs_mogrify_doc},
-    {"fileno", (PyCFunction)psyco_curs_fileno,
-     METH_VARARGS, psyco_curs_fileno_doc},
-    {"isready", (PyCFunction)psyco_curs_isready,
-     METH_VARARGS, psyco_curs_isready_doc},
     {"copy_from", (PyCFunction)psyco_curs_copy_from,
      METH_VARARGS|METH_KEYWORDS, psyco_curs_copy_from_doc},
     {"copy_to", (PyCFunction)psyco_curs_copy_to,
@@ -1627,7 +1553,7 @@ static struct PyMemberDef cursorObject_members[] = {
     {"rowcount", T_LONG, OFFSETOF(rowcount), RO,
         "Number of rows read from the backend in the last command."},
     {"arraysize", T_LONG, OFFSETOF(arraysize), 0,
-        "Number of records `fetchmany()` must fetch if not explicitely " \
+        "Number of records `fetchmany()` must fetch if not explicitly " \
         "specified."},
     {"description", T_OBJECT, OFFSETOF(description), RO,
         "Cursor description as defined in DBAPI-2.0."},
