@@ -227,6 +227,32 @@ psyco_lobj_get_closed(lobjectObject *self, void *closure)
     return closed;
 }
 
+#if PG_VERSION_HEX >= 0x080300
+
+#define psyco_lobj_truncate_doc \
+"truncate(len=0) -- Truncate large object to given size."
+
+static PyObject *
+psyco_lobj_truncate(lobjectObject *self, PyObject *args)
+{
+    int len = 0;
+
+    if (!PyArg_ParseTuple(args, "|i", &len))
+        return NULL;
+
+    EXC_IF_LOBJ_CLOSED(self);
+    EXC_IF_LOBJ_LEVEL0(self);
+    EXC_IF_LOBJ_UNMARKED(self);
+
+    if (lobject_truncate(self, len) < 0)
+        return NULL;
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+#endif /* PG_VERSION_HEX >= 0x080300 */
+
 
 /** the lobject object **/
 
@@ -247,6 +273,10 @@ static struct PyMethodDef lobjectObject_methods[] = {
      METH_VARARGS, psyco_lobj_unlink_doc},
     {"export",(PyCFunction)psyco_lobj_export,
      METH_VARARGS, psyco_lobj_export_doc},
+#if PG_VERSION_HEX >= 0x080300
+    {"truncate",(PyCFunction)psyco_lobj_truncate,
+     METH_VARARGS, psyco_lobj_truncate_doc},
+#endif /* PG_VERSION_HEX >= 0x080300 */
     {NULL}
 };
 
