@@ -1,22 +1,26 @@
 /* typecast_datetime.c - date and time typecasting functions to python types
  *
- * Copyright (C) 2001-2003 Federico Di Gregorio <fog@debian.org>
+ * Copyright (C) 2001-2010 Federico Di Gregorio <fog@debian.org>
  *
- * This file is part of the psycopg module.
+ * This file is part of psycopg.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2,
- * or (at your option) any later version.
+ * psycopg2 is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published
+ * by the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * In addition, as a special exception, the copyright holders give
+ * permission to link this program with the OpenSSL library (or with
+ * modified versions of OpenSSL that use the same license as OpenSSL),
+ * and distribute linked combinations including the two.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You must obey the GNU Lesser General Public License in all respects for
+ * all of the code used other than OpenSSL.
+ *
+ * psycopg2 is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ * License for more details.
  */
 
 #include <math.h>
@@ -130,15 +134,11 @@ typecast_PYDATETIME_cast(const char *str, Py_ssize_t len, PyObject *curs)
             Dprintf("typecast_PYDATETIME_cast: UTC offset = %ds", tz);
 
             /* The datetime module requires that time zone offsets be
-               a whole number of minutes, so fail if we have a time
-               zone with a seconds offset.
-             */
-            if (tz % 60 != 0) {
-                PyErr_Format(PyExc_ValueError, "time zone offset %d is not "
-                             "a whole number of minutes", tz);
-                return NULL;
-            }
-            tzinfo = PyObject_CallFunction(tzinfo_factory, "i", tz / 60);
+               a whole number of minutes, so truncate the seconds to the
+               closest minute. */
+            // printf("%d %d %d\n", tz, tzmin, round(tz / 60.0));
+            tzinfo = PyObject_CallFunction(tzinfo_factory, "i",
+                (int)round(tz / 60.0));
         } else {
             Py_INCREF(Py_None);
             tzinfo = Py_None;
@@ -188,15 +188,10 @@ typecast_PYTIME_cast(const char *str, Py_ssize_t len, PyObject *curs)
         Dprintf("typecast_PYTIME_cast: UTC offset = %ds", tz);
 
         /* The datetime module requires that time zone offsets be
-           a whole number of minutes, so fail if we have a time
-           zone with a seconds offset.
-        */
-        if (tz % 60 != 0) {
-            PyErr_Format(PyExc_ValueError, "time zone offset %d is not "
-                         "a whole number of minutes", tz);
-            return NULL;
-        }
-        tzinfo = PyObject_CallFunction(tzinfo_factory, "i", tz / 60);
+           a whole number of minutes, so truncate the seconds to the
+           closest minute. */
+        tzinfo = PyObject_CallFunction(tzinfo_factory, "i",
+            (int)round(tz / 60.0));
     } else {
         Py_INCREF(Py_None);
         tzinfo = Py_None;
